TeamEquipCompare = LibStub("AceAddon-3.0"):NewAddon("TeamEquipCompare", "AceConsole-3.0", "AceComm-3.0", "AceEvent-3.0");
local tec = TeamEquipCompare;
tec.version = "0.3";
tec.commPrefix = "TEC_COMMS";
tec.party = {};

local defaultDB = {
	global = {
		version = tec.version,
		enabled = true,
		showDebugMsgs = false,
		showNotificationMsgs = true,
		requireCtrlKey = true,
		hideUnusable = true,
		levelCheck = false,
		trackMoney = true,
		moneyThreshold = 5,
	}
};

function tec:notify(msg)
	if (msg) and (tec.db.global.showNotificationMsgs) then
		self:Print(msg)
	end
end

function tec:debug(msg)
	if (msg) and (tec.db.global.showDebugMsgs) then
		self:Print("[DEBUG]: "..msg)
	end
end

function tec:OnInitialize()
	-- load global saved variable, and update with new values if needed
	tec.db = LibStub("AceDB-3.0"):New("TEC_DB", defaultDB, true);
	if (tec.db.version ~= defaultDB.version) then
		tec:notify("Version difference found, setting new values to default...");
		tec.db = TEC_UpdateSavedDB(tec.db, defaultDB);
	end

	-- register communication channel
	self:RegisterComm(tec.commPrefix, "OnCommReceived");

	-- register events
	self:RegisterEvent("PLAYER_LOGIN", "OnPlayerLogin");
	self:RegisterEvent("GROUP_ROSTER_UPDATE", "OnGroupRosterUpdate");
	self:RegisterEvent("PLAYER_EQUIPMENT_CHANGED", "OnPlayerEquipmentChanged");
	self:RegisterEvent("PLAYER_LEVEL_UP", "OnPlayerLevelUp");
	self:RegisterEvent("PLAYER_MONEY", "OnPlayerMoney");
	self:RegisterEvent("PLAYER_SPECIALIZATION_CHANGED", "OnPlayerSpecializationChanged");

	-- register command options, and add tab to Blizzards Interface/AddOns GUI
	LibStub("AceConfig-3.0"):RegisterOptionsTable("TeamEquipCompare", self.options, "tec");
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("TeamEquipCompare", "Team Equip Compare");	

	self:Print(format("TeamEquipCompare v%s loaded", tec.version));
end

-- TODO: do I need to handle enabling/disabling addon?
function tec:OnEnable()
	self:Print(format("TeamEquipCompare v%s enabled", tec.version));
end

function tec:OnDisable()
	tec.db.global.enabled = false;
	self:Print(format("TeamEquipCompare v%s disabled", tec.version));
end

function tec:OnPlayerLogin(...)
	-- set initial money value
	tec.lastMoneyUpdate = GetMoney() or 0;
	tec.initialMoney = tec.lastMoneyUpdate;
end

function tec:OnGroupRosterUpdate(...)
	self:debug("OnGroupRosterUpdate Fired");

	-- clear data if not grouped
	if (not IsInGroup()) then
		self:debug("Clearing party data...");
		tec.party = {};

	-- otherwise, update the data as needed
	else
		local unitsToAdd, unitsToRemove = TEC_CheckTeamMembers();

		-- add any new party members
		if (#unitsToAdd > 0) then
			for ii=1, #unitsToAdd do
				-- add key to party table to prevent multiple requests
				tec.party[unitsToAdd[ii]] = { player = { name = unitsToAdd[ii] } };
				TEC_RequestUnitData(unitsToAdd[ii]);
			end
		end

		-- remove orphaned party members
		if (#unitsToRemove > 0) then
			for ii=1, #unitsToRemove do
				self:notify(format("Removing data for '%s'...", unitsToRemove[ii]));
				tec.party[unitsToRemove[ii]] = nil;
			end
		end
	end
end

function tec:OnPlayerEquipmentChanged(event, slotid, ...)
	self:debug(format("PlayerEquipmentChanged Event Fired, slotid: %s", tostring(slotid)));

	-- report my data to team if my equipment changed
	if (IsInGroup()) then
		TEC_SendPlayerData("PARTY");
	end
end

function tec:OnPlayerLevelUp(event, level, ...)
	self:debug(format("OnPlayerLevelUp Event Fired: new level (%s): ", tostring(level)));

	-- report my data to team when my level changes
	if (IsInGroup() and (level)) then
		TEC_SendLevelUpdate(level);
	end
end

function tec:OnPlayerMoney(...)
	self:debug("PlayerMoney Event Fired");
	
	if (tec.db.global.trackMoney) and (IsInGroup()) then
		-- get current player money
		local money = GetMoney();

		-- test to see if the amount has crossed the threshold (in gold), up or down
		if (abs(money - tec.lastMoneyUpdate) > (tec.db.global.moneyThreshold * 10000)) then
			-- send update to team
			TEC_SendMoneyUpdate(money);

			-- update static value
			tec.lastMoneyUpdate = money;
		end
	end
end

function tec:OnPlayerSpecializationChanged(event, unit, ...)
	self:debug(format("OnPlayerSpecializationChanged Event Fired: %s, %s", tostring(event), tostring(unit)));

	-- report my spec to the team if it changes
	if (IsInGroup() and unit == 'player') then
		TEC_SendSpecUpdate(TEC_GetSpecName());
	end
end

function tec:OnCommReceived(_, commMsg, _, unit, ...)
	self:debug(format("OnCommReceived -- %s: %s", unit, tostring(commMsg)));

	-- request to send player data to the party
	if (commMsg == "SYNC") then
		TEC_SendPlayerData("PARTY");

	-- request to send player data to the party
	elseif (commMsg == "REQUEST") then
		TEC_SendPlayerData(unit);

	-- receive level update
	elseif (strfind(commMsg, "LEVEL")) then
		local level = strmatch(commMsg, "LEVEL=(.*)");
		self:notify(format("Received level update from '%s' (%s)", unit, level));

		if (tec.party[unit]) then
			tec.party[unit].level = tonumber(level);
		else
			TEC_RequestUnitData(unit);
		end
		
	-- receive money update
	elseif (strfind(commMsg, "MONEY")) then
		local money = strmatch(commMsg, "MONEY=(.*)");
		self:debug(format("Received money update from '%s' (%s)", tostring(unit), tostring(money)));

		if (tec.party[unit]) then
			tec.party[unit].money = tonumber(money);
		else
			TEC_RequestUnitData(unit);
		end

	-- receive spec update
	elseif (strfind(commMsg, "SPEC")) then
		local spec = strmatch(commMsg, "SPEC=(.*)");
		self:debug(format("Received spec update from '%s' (%s)", tostring(unit), tostring(spec)));

		if (tec.party[unit]) then
			tec.party[unit].spec = spec;
		else
			TEC_RequestUnitData(unit);
		end

	-- otherwise, receive team member data
	else
		local data = TEC_UnpackageData(commMsg);
		if (data) and (data ~= "") then
			tec.party[unit] = data;
			self:notify(format("Received update from '%s'", unit));
		else
			self:notify(format("Error receiving data from %s", unit));
		end
	end
end